/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB_copyright.h.
 * Restrictions in GB_copyright.h.
 *
 *  orbit.c -- display orbits of planets (graphic representation)
 *
 * OPTIONS
 *  -p : If this option is set, ``orbit'' will not display planet names.
 *
 *  -S : Do not display star names.
 *
 *  -s : Do not display ships.
 *
 *  -(number) : Do not display that #'d ship or planet (in case it obstructs
 * 		the view of another object)
 */

#include "GB_copyright.h"
#define EXTERN extern
#include "vars.h"
#include "ships.h"
#include "races.h"
#include <curses.h>
#include <stdio.h>
extern char Shipltrs[];
float Lastx, Lasty, Zoom;
int SCALE = 100;
int orbit_racedata;

orbit(Playernum, APcount, argn, args)
int Playernum;
int APcount;
int argn;
char args[MAXARGS][COMMANDSIZE];
{
register int sh,i,iq;
int DontDispNum= -1, t, flag;
planettype *p;
shiptype *s;
placetype where,shipwhere;
boolean err, DontDispPlanets, DontDispShips, DontDispStars;
char str[PLACENAMESIZE];
int orbit_shdata,orbit_pdata,orbit_stardata;
char buf[1024];
hugestr output;

 DontDispPlanets = DontDispShips = DontDispStars = 0;

 free(Race);
openracedata(&orbit_racedata);
getrace(orbit_racedata, &Race, Playernum);
close_file(orbit_racedata);

 /* find options, set flags accordingly */
 for (flag=1; flag<=argn-1; flag++)
    if (*args[flag]=='-') {
      for (i=1; args[flag][i]!='\0'; i++)
	 switch (args[flag][i]) {
	    case 's': DontDispShips = 1;
		      break;
	    case 'S': DontDispStars = 1;
		      break;
	    case 'p': DontDispPlanets = 1;
		      break;
       	    default : if (sscanf(args[flag]+1,"%d",&DontDispNum)!=1) {
			notify(Playernum, "Bad number %s.\n",args[flag]+1);
			DontDispNum = -1;
		      }
		      if (DontDispNum)
		        DontDispNum--;	/* make a '1' into a '0' */
		      break;
	  }
    }

 if (argn==1) {
   where = Getplace(Playernum, ":",0);
	i = (Dir[Playernum-1].level==LEVEL_UNIV); 
   Lastx = Dir[Playernum-1].lastx[i];
   Lasty = Dir[Playernum-1].lasty[i];
   Zoom = Dir[Playernum-1].zoom[i];
 } else {
   where = Getplace(Playernum, args[argn-1],0);
   Lastx = Lasty = 0.0;
   Zoom = 1.1;
 }

 if (where.err) {
   notify(Playernum, "orbit: error in args.\n");
   return;
 }

	/* orbit type of map */
	sprintf(output, "#");

  switch (where.level) {
    case LEVEL_UNIV:
      for (i=0; i<Sdata.numstars; i++)
	 if (DontDispNum!=i) {
	   DispStar(Playernum, LEVEL_UNIV,Stars[i],DontDispStars, Race->God, buf);
		str_cat(output, buf);
		}

      if (!DontDispShips && (sh = Sdata.ships)) {
       openshdata(&orbit_shdata);

       while (sh) {
	   (void)getship(orbit_shdata, &s, sh);
	   if (DontDispNum != sh) {
	     DispShip(Playernum, &where, s, sh, NULL, Race->God, buf );
		str_cat(output, buf);
		}
	   sh = s->nextship;
	   free(s);
       }

       close_file(orbit_shdata);
      }
      break;

   case LEVEL_STAR:
      DispStar(Playernum, LEVEL_STAR, Stars[where.snum], DontDispStars, Race->God, buf);
	str_cat(output, buf);

      openpdata(&orbit_pdata);


      for (i=0; i<Stars[where.snum]->numplanets; i++)
	 if (DontDispNum!=i) {
	   getplanet(orbit_pdata,&p,Stars[where.snum]->planetpos[i]);
	   DispPlanet(Playernum, LEVEL_STAR,p,Stars[where.snum]->pnames[i],DontDispPlanets, 
				Race->God, buf);
		str_cat(output, buf);
	   free(p);
	 }
 
      close_file(orbit_pdata);

/* check to see if you have ships at orbiting the star, if so you can
see enemy ships */
	iq = 0;
	if(sh = Stars[where.snum]->ships){
	openshdata(&orbit_shdata);
	while (sh && !iq) {
	   (void)getship(orbit_shdata, &s, sh);
		if(s->owner == Playernum && Shipdata[s->type][ABIL_SIGHT])
			iq = 1; /* you are there to sight, need a crew */
		sh = s->nextship;
		free(s);
		}
       close_file(orbit_shdata);
	}


      if (!DontDispShips && (sh = Stars[where.snum]->ships)) {
       openshdata(&orbit_shdata);

       while (sh) {
	   (void)getship(orbit_shdata, &s, sh);
	   if (DontDispNum != sh && !(s->owner != Playernum && s->type == STYPE_MINE) ) {

		if((s->owner == Playernum) || (iq == 1)) {
	     DispShip(Playernum, &where, s, sh, NULL, Race->God, buf);
		str_cat(output, buf);
		}
		}
	   sh = s->nextship;
	   free(s);
       }
       close_file(orbit_shdata);
      }
      break;

   case LEVEL_PLAN:

      openpdata(&orbit_pdata);
      getplanet(orbit_pdata,&p,Stars[where.snum]->planetpos[where.pnum]);
      close_file(orbit_pdata);
      DispPlanet(Playernum, LEVEL_PLAN, p, Stars[where.snum]->pnames[where.pnum],
			 DontDispPlanets, Race->God, buf);
	str_cat(output, buf);

/* check to see if you have ships at landed or
	orbiting the planet, if so you can see orbiting enemy ships */
	iq = 0;

	if(sh = p->ships){
	openshdata(&orbit_shdata);
	while (sh && !iq) {
	   (void)getship(orbit_shdata, &s, sh);
		if(s->owner == Playernum && Shipdata[s->type][ABIL_SIGHT])
			iq = 1; /* you are there to sight, need a crew */
		sh = s->nextship;
		free(s);
		}
       close_file(orbit_shdata);
	}

/* end check */

      if (!DontDispShips && (sh = p->ships)) {
       openshdata(&orbit_shdata);
       while (sh) {
	   (void)getship(orbit_shdata, &s, sh);
	   if (DontDispNum != sh) {
	     	if( !(s->is_docked && s->whatdest==LEVEL_PLAN) ) {
		if((s->owner == Playernum) || (iq ==1)) {
	 		    DispShip(Playernum, &where, s, sh, p, Race->God, buf);
			str_cat(output, buf);
			}
			}
		}
	   sh = s->nextship;
	   free(s);
       }
       close_file(orbit_shdata);
      }
      free(p);
      break;

   default:
      notify(Playernum,"Bad scope.\n");
	return;	 
  }

	notify(Playernum, output);
}


DispStar(Playernum, level, star, DontDispStars, God, string)
int Playernum;
int level, God;
startype *star;
int DontDispStars;
char *string;
{
planettype *p;
shiptype *s;
 int x,y,i,col,orbit_pdata;
 int populated, stand;
 int iq, orbit_shdata, sh;
 register int x2,y2;
 float fac;
 char *c;
char temp[1024];

*string = 0; 

if (level==LEVEL_UNIV) {
   fac = 1.0;
   x = (int)(SCALE+((SCALE*(star->xpos-Lastx))/(UNIVSIZE*Zoom)));
   y = (int)(SCALE+((SCALE*(star->ypos-Lasty))/(UNIVSIZE*Zoom)));
 } else if (level==LEVEL_STAR) {
   fac = 1000.0;
   x = (int)(SCALE+(SCALE*(-Lastx))/(SYSTEMSIZE*Zoom));
   y = (int)(SCALE+(SCALE*(-Lasty))/(SYSTEMSIZE*Zoom));
 }
/*
 if (star->nova_stage)
	DispArray(x, y, 11,7, Novae[star->nova_stage-1], fac);
 else */
if (y>=0 && x>=0) {

	iq = 0;
	/*
	if (isset(star->inhabited, Playernum)) {
		sh = star->ships;
       		openshdata(&orbit_shdata);
       		while (sh && !iq) {
	   		(void)getship(orbit_shdata, &s, sh);
			if(s->owner == Playernum)
				iq = 1;
		sh = s->nextship;
	   	free(s);
       		}
       		close_file(orbit_shdata);

	}
       */
		if (isset(star->explored,Playernum)) stand=1;
			else stand = 0;	
	sprintf(temp, "%d %d %d 0 * ",stand,x,y);
	str_cat(string, temp);

/* check to see if the any planets are populated here. If so,
	standout() the name of it */

	sprintf(temp, "%d %s ;",isset(star->inhabited, Playernum), star->name);
	str_cat(string, temp);
	   }
}

DispPlanet(Playernum, level, p, name, DontDispPlanets, God, string)
int Playernum;
int level, God;
planettype *p;
char *name;
int DontDispPlanets;
char *string;
{
int x,y;
int stand;
char temp[1024];

*string = 0;

 if (level==LEVEL_STAR) {
   y = (int)(SCALE+(SCALE*(p->ypos-Lasty))/(SYSTEMSIZE*Zoom));
   x = (int)(SCALE+(SCALE*(p->xpos-Lastx))/(SYSTEMSIZE*Zoom));
 } else if (level==LEVEL_PLAN) {
   y = (int)(SCALE+(SCALE*(-Lasty))/(PLORBITSIZE*Zoom));
   x = (int)(SCALE+(SCALE*(-Lastx))/(PLORBITSIZE*Zoom));
 }
 if (x>=0 && y>=0) {

		if(p->info[Playernum-1].explored)stand=1;
			else stand = 0;

	sprintf(temp, "%d %d %d 0 @ ",stand,x,y);
	str_cat(string, temp);

	if(p->info[Playernum-1].numsectsowned)stand=1;
		else stand=0;

	sprintf(temp, "%d %s;",stand, name);
	str_cat(string, temp);

 }
}



DispShip(Playernum, where, ship, shipno, pl , God, string)
int Playernum;
placetype *where;
shiptype *ship;
int shipno, God;
planettype *pl;
char *string;
{
 int x,y,x2,y2, xa, xb, wm,pdata,shipdata;
 int stand;
 shiptype *aship;
 planettype *apl;
 float xt,yt,slope;
 char *c, temp[1024];

*string = 0;

 if (!ship->is_alive)
	return;

 switch (where->level) {
	case LEVEL_PLAN:
		x = (int)(SCALE + (SCALE*(ship->xpos-(Stars[where->snum]->xpos+pl->xpos) - Lastx))
			/(PLORBITSIZE*Zoom));
		y = (int)(SCALE + (SCALE*(ship->ypos-(Stars[where->snum]->ypos+pl->ypos) - Lasty))
			/(PLORBITSIZE*Zoom));
		break;
	case LEVEL_STAR:
		x = (int)(SCALE + (SCALE*(ship->xpos-Stars[where->snum]->xpos - Lastx))
			/(SYSTEMSIZE*Zoom));
		y = (int)(SCALE + (SCALE*(ship->ypos-Stars[where->snum]->ypos - Lasty))
			/(SYSTEMSIZE*Zoom));
		break;
	case LEVEL_UNIV:
	     	x = (int)(SCALE + (SCALE*(ship->xpos-Lastx))/(UNIVSIZE*Zoom));
		y = (int)(SCALE + (SCALE*(ship->ypos-Lasty))/(UNIVSIZE*Zoom));
		break;
	default:
		notify(Playernum, "WHOA! error in DispShip.\n");
		return;
 }

 switch (ship->type) {

  case STYPE_MIRROR:
    if (ship->aimed_at.level==LEVEL_STAR) {
	xt = Stars[ship->aimed_at.snum]->xpos;
	yt = Stars[ship->aimed_at.snum]->ypos;
    } else if (ship->aimed_at.level==LEVEL_PLAN) {
	if (where->level==LEVEL_PLAN && 
			ship->aimed_at.pnum == where->pnum) {  
				/* same planet */
		xt = Stars[ship->aimed_at.snum]->xpos + pl->xpos;
		yt = Stars[ship->aimed_at.snum]->ypos + pl->ypos;
	} else {	/* different planet */
		openpdata(&pdata);
		getplanet(pdata,&apl,Stars[where->snum]->planetpos[where->pnum]);
		close_file(pdata);
		xt = Stars[ship->aimed_at.snum]->xpos + apl->xpos;
		yt = Stars[ship->aimed_at.snum]->ypos + apl->ypos;
		free(apl);
	}
    } else if (ship->aimed_at.level==LEVEL_SHIP) {
	openshdata(&shipdata);
	if (getship(shipdata,&aship,ship->aimed_at.shipno)) {
		xt = aship->xpos;
		yt = aship->ypos;
	} else
		xt = yt = 0.0;
	close_file(shipdata);
	free(aship);
    } else
	xt = yt = 0.0;

    wm=0;

    if(xt == ship->xpos){
		wm=0;
		if(yt > ship->ypos)wm=4;
	} else {
		slope = (yt - ship->ypos) / (xt - ship->xpos);
		if(yt == ship->ypos){
			wm=6;
			if(xt > ship->xpos)wm=2;
			}

		if(yt > ship->ypos) {
			if(slope < -2.414)wm=4;
			if(slope > -2.414)wm=5;
			if(slope > -0.414)wm=6;
			if(slope >  0.000)wm=2;
			if(slope >  0.414)wm=3;
			if(slope >  2.414)wm=4;
			}
		if(yt < ship->ypos) {
			if(slope < -2.414)wm=0;
			if(slope > -2.414)wm=1;
			if(slope > -0.414)wm=2;
			if(slope >  0.000)wm=6;
			if(slope >  0.414)wm=7;
			if(slope >  2.414)wm=0;
			}
	}

				/* (magnification) */
    if (x>=0 && y>=0) {
	if(ship->owner==Playernum)stand=1;
	else stand = 0;

	sprintf(string, "%d %d %d %d %c %d %d;",stand,x,y,wm,Shipltrs[ship->type],stand,shipno);
    }
    break;

  case OTYPE_CANIST:
    break;

  case OTYPE_VN: { 
		register int n,xa,ya;
		   register float fac;
	wm = 0;
	/* make a cloud of Von Neumann machines */
    if (ship->whatorbits!=LEVEL_UNIV || (ship->owner == Playernum || God)) {
	fac = ship->object.number / 
		((ship->whatorbits==LEVEL_UNIV ? 100.0 : 
		(ship->whatorbits==LEVEL_STAR ? 30.0 : 4.0)) * Zoom);
	for (n=1; n<=ship->object.number && n<267; n++) {
		xa = int_rand(x - (int)fac, x + (int)fac);
		ya = int_rand(y - (int)fac, y + (int)fac);
		if (xa>=0 && ya>=0) {
		if(ship->owner==Playernum)stand=1;
			else stand = 0;
	sprintf(temp, "%d %d %d %d %c %d %d;",stand,xa,ya,wm,Shipltrs[ship->type],
				stand,shipno);

	str_cat(string, temp);
		}
     	}
    }
   }
   break;

  default:
	/* other ships can only be seen when in system */
	wm=0;
    if (ship->whatorbits!=LEVEL_UNIV || (ship->owner == Playernum || God))
       if (x>=0 && y>=0) {
		if(ship->owner==Playernum)stand=1;
			else stand = 0;
	sprintf(string, "%d %d %d %d %c %d %d;",stand,x,y,wm,Shipltrs[ship->type],stand,shipno);

       }
    break;
 }


}



